import datetime


# Максимальная дата в данных, которые приходят из базы ГАРа
# Объекты, имеющие эту дату в поле даты окончания действия, считаются актуальными
GAR_DATE_MAX = datetime.date(2079, 6, 6)
